//===========================================================================================
// medDeathScreamDampener | a UT 99 mutator | by Helen
//
// - Allows you to tone down or remove the death screams of players and bots.
// - The type of sound level to choose is one of DEATHCRY_None, DEATHCRY_Low, or DEATHCRY_Normal.
// - The default value is DEATHCRY_Low.
//
// == COMMAND LINE ==
// Add mutator=medDeathScreamDampener110.medDeathScreamDampener to your command line parameter list.
//
// == SERVER ==
// Add the medDeathScreamDampener.ini to the System directory.
// If you want the sound value to be other than Low, open the file, and for the ServerDeathCryType
// setting, change the DEATHCRY_Low value to DEATHCRY_None (or you could set it to DEATHCRY_Normal,
// but then there is really no point in running this mutator).
// Note that if you were to open the medDeathScreamDampener.ini after running the mutator once, a LocalDeathCryType
// variable will appear. Don't bother trying to assign anything to it, it will have no effect on the server.
//
// == CLIENT ==
// The player who joins will mostly have to hear what the server has specified. But if
// they really want to hear a different sound level for just themself, they can do this:
// - Join the server running the mutator for a few seconds, then disconnect.
// - Go to the System folder, and find and open the medDeathScreamDampener.ini file.
// - Change the value of the LocalDeathCryType to one of DEATHCRY_None, DEATHCRY_Low, or DEATHCRY_Normal.
// Note that the player will be able to see the ServerDeathCryType variable in
// there, but setting a value for it will have no effect on the server's value.
//
//===========================================================================================

class medDeathScreamDampener expands Mutator config(medDeathScreamDampener);

enum EDeathCryType
{
	DEATHCRY_Unassigned,   // 0
	DEATHCRY_None,         // 1
	DEATHCRY_Low,          // 2
	DEATHCRY_Normal        // 3
};

var() config string ServerDeathCryType;
var() config string LocalDeathCryType;

var EDeathCryType mServerDeathCryType;
var EDeathCryType mLocalDeathCryType;
var bool bSaved;

replication
{
    // Send this value from server to client,
    reliable if (Role == ROLE_Authority)
        mServerDeathCryType;
}

simulated function PreBeginPlay()
{
    if (Role == ROLE_Authority)
	{
        if (ServerDeathCryType == "DEATHCRY_None")
        {
            mServerDeathCryType = DEATHCRY_None;
        }
        else if (ServerDeathCryType == "DEATHCRY_Low")
        {
            mServerDeathCryType = DEATHCRY_Low;
        }
        else if (ServerDeathCryType == "DEATHCRY_Normal")
        {
            mServerDeathCryType = DEATHCRY_Normal;
        }
        else
        {
            // This is essentially the default value.
            mServerDeathCryType = DEATHCRY_Low;
            ServerDeathCryType = "DEATHCRY_Low";
        }
    }
    else if (Role == ROLE_SimulatedProxy)
	{
        if (LocalDeathCryType == "DEATHCRY_None")
        {
            mLocalDeathCryType = DEATHCRY_None;
        }
        else if (LocalDeathCryType == "DEATHCRY_Low")
        {
            mLocalDeathCryType = DEATHCRY_Low;
        }
        else if (LocalDeathCryType == "DEATHCRY_Normal")
        {
            mLocalDeathCryType = DEATHCRY_Normal;
        }
        else
        {
            if (mServerDeathCryType != DEATHCRY_Unassigned)
            {
                mLocalDeathCryType = mServerDeathCryType;
                LocalDeathCryType = string(GetEnum(Enum'EDeathCryType', mLocalDeathCryType));
            }
            else
            {
                // The server didn't pass on any value either,
                // so this is essentially the default value.
                mLocalDeathCryType = DEATHCRY_Low;
                LocalDeathCryType = "DEATHCRY_Low";
            }
        }
	}

    // Save the config files.
    if (!bSaved)
    {
        SaveConfig();
        bSaved = true;
    }
}

function ModifyPlayer(Pawn Owner)
{
    /*  ModifyPlayer() is only called on the server. Bots are not replicated
        to the client, so this will completely take care of the Bot's sounds.
        However, at this point, the players are already replicated to the client.
        So while this will cover the player's server sounds, we still have to
        take care of the player's client sounds. This occurs later in Tick(). */

    /*  If the sounds requested are normal, we do not have to do
        anything, as they are already in the 'normal' state. */
    if (mServerDeathCryType > DEATHCRY_Low)
        return;

    SetDeathCryLevel(Owner, mServerDeathCryType);

	if ( NextMutator != None )
		NextMutator.ModifyPlayer(Owner);
}

simulated function Tick(float Delta)
{
	local TournamentPlayer aPlayer;
    local Bot aBot;
	local int i;

    if (Role == ROLE_Authority)
	{
        // We have already taken care of the server.
		Disable('Tick');
		return;
	}

    // Note that we don't have to set the sounds for the bots on the client.
    // This is because they are not replicated to the client, so the client
    // will always hear the Bot sound that is set at the server level.
    

	foreach AllActors(Class'TournamentPlayer', aPlayer)
	{
        SetDeathCryLevel(aPlayer, mLocalDeathCryType);
	}

    // Just one iteration of Tick seems to do the trick, so...
	Disable('Tick');
}

simulated function SetDeathCryLevel(Pawn APawn, EDeathCryType ADeathCryLevel)
{
    local int i;
    local Bot aBot;
    local TournamentPlayer aPlayer;
    local sound aDeathCry;
    local sound aDeath0;
    local sound aDeath1;
    local sound aDeath2;
    local sound aDeath3;
    local sound aDeath4;
    local sound aDeath5;

    if(APawn == None)
        return;

    if (ADeathCryLevel == DEATHCRY_Unassigned)
        return;

    // Ensure we are dealing with a player or a bot.
    if((!APawn.IsA('TournamentPlayer')) && (!APawn.IsA('Bot')))
        return;

    // Prep the sound to use for the death cry.
    if (ADeathCryLevel == DEATHCRY_None)
    {
        // A quiet death.
        aDeathCry = None;
    }
    else if (ADeathCryLevel == DEATHCRY_Low)
    {
        // A lesser sound of discomfort.
        aDeathCry = APawn.HitSound1;
    }
    else if (ADeathCryLevel == DEATHCRY_Normal)
    {
        // Prep some sounds to restore.

        if(APawn.IsA('TBoss'))
        {
            aDeath0 = Sound'Botpack.Boss.BDeath1';
            aDeath1 = Sound'Botpack.Boss.BDeath1';
            aDeath2 = Sound'Botpack.Boss.BDeath3';
            aDeath3 = Sound'Botpack.Boss.BDeath4';
            aDeath4 = Sound'Botpack.Boss.BDeath3';
            aDeath5 = Sound'Botpack.Boss.BDeath4';
        }
        else if(APawn.IsA('Skaarj'))
        {
            aDeath0 = Sound'UnrealI.Skaarj.SKPDeath1';
            aDeath1 = Sound'UnrealI.Skaarj.SKPDeath2';
            aDeath2 = Sound'UnrealI.Skaarj.SKPDeath3';
            aDeath3 = Sound'UnrealI.Skaarj.SKPDeath3';
            aDeath4 = Sound'UnrealI.Skaarj.SKPDeath2';
            aDeath5 = Sound'UnrealI.Skaarj.SKPDeath1';
        }
        else if((APawn.IsA('TournamentMale')) || (APawn.IsA('MaleBotPlus')))
        {
            aDeath0 = Sound'Botpack.deathc1';
            aDeath1 = Sound'Botpack.deathc51';
            aDeath2 = Sound'Botpack.deathc3';
            aDeath3 = Sound'Botpack.deathc4';
            aDeath4 = Sound'Botpack.deathc53';
            aDeath5 = Sound'Botpack.deathc53';
        }
        else if ((APawn.IsA('TournamentFemale')) || (APawn.IsA('FemaleBotPlus')))
        {
            aDeath0 = Sound'Botpack.death1d';
            aDeath1 = Sound'Botpack.death2a';
            aDeath2 = Sound'Botpack.death3c';
            aDeath3 = Sound'Botpack.decap01';
            aDeath4 = Sound'Botpack.death41';
            aDeath5 = Sound'Botpack.death42';
        }
    }

    // Assign the sound.
    if(APawn.IsA('TournamentPlayer'))
    {
        aPlayer = TournamentPlayer(APawn);

        if (ADeathCryLevel == DEATHCRY_Normal)
        {
            aPlayer.Deaths[0] = aDeath0;
            aPlayer.Deaths[1] = aDeath1;
            aPlayer.Deaths[2] = aDeath2;
            aPlayer.Deaths[3] = aDeath3;
            aPlayer.Deaths[4] = aDeath4;
            aPlayer.Deaths[5] = aDeath5;
        }
        else
        {
            for(i=0; i<6; i++)
            {
                aPlayer.Deaths[i] = aDeathCry;
            }
        }
    }
    else
    {
        aBot = Bot(APawn);

        if (ADeathCryLevel == DEATHCRY_Normal)
        {
            aBot.Deaths[0] = aDeath0;
            aBot.Deaths[1] = aDeath1;
            aBot.Deaths[2] = aDeath2;
            aBot.Deaths[3] = aDeath3;
            aBot.Deaths[4] = aDeath4;
            aBot.Deaths[5] = aDeath5;
        }
        else
        {
            for(i=0; i<6; i++)
            {
                aBot.Deaths[i] = aDeathCry;
            }
        }
    }
}

defaultproperties
{
    bAlwaysRelevant=true;   // Without this assignment, this whole mutator does not work.
    RemoteRole=ROLE_SimulatedProxy;
    bSaved=false;
    mServerDeathCryType=DEATHCRY_Unassigned;
    mLocalDeathCryType=DEATHCRY_Unassigned;
}
